@@ -114,9 +114,17 @@ def client_from_dict_config(config, client=None, clear_global=False):
     disabled_timers
       Sequence of string tokens identifying timers that are to be deactivated.
     filters
-      Sequence of 2-tuples `(callable, config)`, where `callable` is dotted
-      notation reference to filter callable and `config` is dictionary config
-      to be passed in during each use of the filter.
+      Sequence of 2-tuples `(filter_provider, config)`. Each `filter_provider`
+      is a dotted name referring to a function which, when called and passed
+      the associated `config` dict as kwargs, will return a usable MetlogClient
+      filter function.
+    plugins
+      Nested dictionary containing plugin configuration. Keys are the plugin
+      names (i.e. the name the method will be given when attached to the
+      client). Values are 2-tuples `(plugin_provider, config)`. Each
+      `plugin_provider` is a dotted name referring to a function which, when
+      called and passed the associated `config`, will return the usable plugin
+      method.
     sender
       Nested dictionary containing sender configuration.
     global
@@ -149,67 +157,46 @@ def client_from_dict_config(config, client=None, clear_global=False):
     logger = config.get('logger', '')
     severity = config.get('severity', 6)
     disabled_timers = config.get('disabled_timers', [])
-    plugin_param = config.pop('plugins', {})
+    filter_specs = config.get('filters', [])
+    plugins_data = config.pop('plugins', {})
     global_conf = config.get('global', {})
-    resolver = DottedNameResolver()
 
+    # update global config stored in CLIENT_HOLDER
     from metlog.holder import CLIENT_HOLDER
     if clear_global:
         CLIENT_HOLDER.global_config = {}
     CLIENT_HOLDER.global_config.update(global_conf)
 
-    filters = [(resolver.resolve(filter_dottedname), filter_config) for
-               (filter_dottedname, filter_config) in config.get('filters', [])]
+    resolver = DottedNameResolver()
 
+    # instantiate sender
     sender_clsname = sender_config.pop('class')
     sender_cls = resolver.resolve(sender_clsname)
     sender_args = sender_config.pop('args', tuple())
     sender = sender_cls(*sender_args, **sender_config)
 
+    # initialize filters
+    filters = [resolver.resolve(dotted_name)(**cfg)
+               for (dotted_name, cfg) in filter_specs]
+
+    # instantiate and/or configure client
     if client is None:
         client = MetlogClient(sender, logger, severity, disabled_timers,
                               filters)
     else:
         client.setup(sender, logger, severity, disabled_timers, filters)
 
-    # Load plugins and pass in config
-    for plugin_name, plugin_config in plugin_param.items():
-        config = plugin_config.pop('plugin.provider')
-        plugin = config(plugin_param[plugin_name])
-        client.add_method(plugin_name, plugin)
+    # initialize plugins and attach to client
+    for plugin_name, plugin_spec in plugins_data.items():
+        # each plugin spec is a 2-tuple: (dotted_name, cfg)
+        plugin_config = plugin_spec[1]
+        plugin_override = plugin_config.pop('override', False)
+        plugin_fn = resolver.resolve(plugin_spec[0])(plugin_config)
+        client.add_method(plugin_name, plugin_fn, plugin_override)
 
     return client
 
 
-def _get_filter_config(config, section):
-    """
-    Extract the various filter configuration sections from the config object
-    and return a filters sequence suitable for passing to the client
-    constructor.
-    """
-    return _get_plugin_config(config, section, 'filter')
-
-
-def _get_plugin_config(config, section, plugin):
-    """
-    Extract the various plugin configuration sections from the config object
-    and return a plugin sequence suitable for passing to the client
-    constructor.
-    """
-    # plugins config
-    plugins_prefix = '%s_%s_' % (section, plugin)
-    plugin_sections = [s for s in config.sections()
-                       if s.startswith(plugins_prefix)]
-    plugins = []
-    for plugin_section in plugin_sections:
-        plugin_config = {}
-        for opt in config.options(plugin_section):
-            plugin_config[opt] = _convert(config.get(plugin_section, opt))
-        plugin_dottedname = plugin_config.pop(plugin)  # 'plugin' key req'd
-        plugins.append((plugin_dottedname, plugin_config))
-    return plugins
-
-
 def dict_from_stream_config(stream, section):
     """
     Parses configuration from a stream and converts it to a dictionary suitable
@@ -222,31 +209,42 @@ def dict_from_stream_config(stream, section):
     config = ConfigParser.SafeConfigParser()
     config.readfp(stream)
     client_dict = {}
+
+    # extract main client configuration
     for opt in config.options(section):
         client_dict[opt] = _convert(config.get(section, opt))
 
-    filters = _get_filter_config(config, section)
-    if filters:
-        client_dict['filters'] = filters
+    # extract filter config from filter sections
+    filters = []
+    filter_sections = [n for n in config.sections()
+                       if n.startswith('%s_filter' % section)]
+    for filter_section in filter_sections:
+        filter_config = {}
+        for opt in config.options(filter_section):
+            if opt == 'provider':
+                # must be a dotted name string, don't convert
+                dotted_name = config.get(filter_section, opt)
+            else:
+                filter_config[opt] = _convert(config.get(filter_section, opt))
+        filters.append((dotted_name, filter_config))
+    client_dict['filters'] = filters
 
-    # Load any plugin configuration
+    # extract plugin config from plugin sections
+    plugins = {}
     plugin_sections = [n for n in config.sections()
                        if n.startswith("%s_plugin" % section)]
-    resolver = DottedNameResolver()
-    plugin_param = {}
     for plugin_section in plugin_sections:
         plugin_name = plugin_section.replace("%s_plugin_" % section, '')
-        plugin_dict = {}
+        plugin_config = {}
+        provider = ''
         for opt in config.options(plugin_section):
             if opt == 'provider':
-                configurator = resolver.resolve(config.get(plugin_section,
-                                                           opt))
-                plugin_dict['plugin.provider'] = configurator
-                continue
-            plugin_dict[opt] = _convert(config.get(plugin_section, opt))
-        plugin_param[plugin_name] = plugin_dict
-    if plugin_param:
-        client_dict['plugins'] = plugin_param
+                # must be a dotted name string, don't convert
+                provider = config.get(plugin_section, opt)
+            else:
+                plugin_config[opt] = _convert(config.get(plugin_section, opt))
+        plugins[plugin_name] = (provider, plugin_config)
+    client_dict['plugins'] = plugins
 
     client_dict = nest_prefixes(client_dict)
     return client_dict
